"""

Use Cases
 each project creates a user defined role on the project
 object and assigns to that role permissions useful for the
 project. the owner and the contributors are then given that
 role, which means that they only have these roles in the 
 context of this object. hmm.. i haven't thought through if 
 this allows weird security violations through some hideous
 acquisition url, but again they only get perms that they would
 in a project context (iotw, make these perms useless elsewhere)
 
 on second thought we'll just grant them these roles as local 
 roles and take advantage of local role security checks.
 
 more thinking, while creating roles on the project does make
 a certain amount of sense in that it offers arbitrary flexiblity
 at a later point for subdividing the number of roles and user 
 mappings to those roles, it is much easier in this setting to 
 create the role up the tree and then create local role mappings
 for users.


"""
from AccessControl import getSecurityManager
from AccessControl.Permission import Permission
from AccessControl.PermissionRole import rolesForPermissionOn
from AccessControl import SecurityManager, ClassSecurityInfo

from Interface import Base

from Products.Gideon.Objects.Namespace import *

class IGroupManagementService(Base):

    """

    Provides a central mechanism for managing
    all groups

    """

#class GroupManagementService(Service):
#    pass

class IGroupManagement(Base):

    """
    Groups provide an abstraction over the local roles
    mechanism, and provide a api for managing groups
    of users on a container object.

    XXX because groups rely on the local roles mechanism
    nested group container objects will allow those
    defined in groups higher up the containment heirarchy
    to have their group roles in affect in sub group
    containers.

    XXX because groups rely on the local roles mechanism
    changes to the containers local roles settings can
    leave the group object in a possibly invalid
    state or with conflict errors during group management
    operations. the function sync, should be called to
    periodically if this is the case... this should
    be explained in more detail.
    
    """

    #one_member_p = Attribute("""
    #    Boolean Value to restrict removal of group's last member
    #    """)

    def add_role(self, role, default=0):
        """
        add a local role that will be given to group
        memebers. if default is true, the role will
        be give to all members of the group.
        """

    def add_user(self, user_id, default_p=1, roles=None):
        """
        add a user to the group, the user will get the
        default roles unless the default_p arg is false.
        additional roles assigned to the group maybe assigned
        to the group member by passing in a tuple of group
        roles.
        """

    def add_role_for_user(self, role, user_id):
        """
        add an additional group role for a group member
        """

    def get_group_members(self):
        """
        return a dictionary of user_id to role mappings
        that represents group members and corresponding roles
        """

    def get_group_roles(self):
        """
        return a dictionary of role to userid
        that represent group roles and corresponding members
        """

    def remove_role(self, role):
        """
        removes a role from the group
        """

    def remove_user(self, user_id):
        """
        remove a user from the group
        """

    def remove_role_from_user(self, role, user_id):
        """
        remove a role from a group member
        """

    def sync(self):
        pass

    def get_container(self):
        """
        get the container that the group object is managing
        local roles for
        """

class GroupManagementError(Exception): pass
class DuplicateRole(GroupManagementError): pass
class InvalidRole(GroupManagementError): pass
class InvalidGroupMember(GroupManagementError): pass
class InvalidUser(GroupManagementError): pass
class LastGroupMember(GroupManagementError): pass

class GroupManagement(SimpleItem):

    meta_type = 'Group Management'

    __implements__ = (IGroupManagement,)

    security = ClassSecurityInfo()

    security.declareObjectPrivate()

    def __init__(self, id='Groups', one_member=1):

        self.id = id
        self._group_roles = PersistentMapping()
        self._group_users = PersistentMapping()
        self._default_roles = ()
        self.one_member_p = one_member
        
    def add_role(self, role, default=0):

        self._group_roles[role]=[]

        container = self.get_container()
        result = container._addRole(role)
        
        if result is not None:
            raise DuplicateRole("%s Role already exists here"%role)

        if default:
            default_roles = list(self._default_roles)

            if role in default_roles:
                return

            default_roles.append(role)
            self._default_roles = tuple(default_roles)
            
            for u in self._group_users.keys():
                self.add_role_for_user(u, role)

    def add_user(self, user_id):

        # XXX should check to make sure they exist :(

        if self._default_roles:
            container = self.get_container()
            container.manage_addLocalRoles(userid, self._default_roles)
            
        self._group_users[user_id]=list(self._default_roles)

    def add_role_for_user(self, role, user_id):

        container = self.get_container()
        valid_roles = container.valid_roles()

        if role not in valid_roles:
            raise InvalidRole("%s Role Does Not Exist"%role)
        
        user_roles = self._group_users.get(user_id, None)

        if user_roles is None:
            raise InvalidGroupMember("%s is not a group member"%user_id)

        user_roles.append(role)
        self._group_users[user_id]=user_roles
        container.manage_addLocalRoles(userid, (role,))
            
    def get_group_members(self):
        
        return copy.copy(self._group_users)

    def get_group_roles(self):
        
        return copy.copy(self._group_roles)

    def remove_user(self, user_id):

        container = self.get_container()
        user_roles = self._group_users.get(user_id, None)
        
        if user_roles is None:
            return

        if len(self._group_users) == 1 and self.one_member_p:
            raise LastGroupMember("Cannot Remove Last Group Member")

        local_roles = container.get_local_roles_for_userid(user_id)

        new_local_roles = []

        for lr in local_roles:
            if lr not in user_roles:
                new_local_roles.append(lr)

        if new_local_roles:
            container.manage_setLocalRoles(user_id, new_local_roles)
        else:
            container.manage_delLocalRoles((user_id,))

        del self._group_users[user_id]
        
    def remove_role(self, role):

        container = self.get_container()
        
        users = self._group_roles.get(role, None)

        for u in users:
            self.remove_role_from_user(role, u)

        if role in self._default_roles:
            self._default_roles=tuple(list(self._default_roles).remove(role))

        ### not sure if we should do this last bit
        ### its all about playing nice with others ;)
        container._delRoles((role,))

    def remove_role_from_user(self, role):

        raise NotImplementedError

        # not finished yet
    
        container = self.get_container()
        user_roles = self._group_users.get(user_id, None)
        
        if user_roles is None:
            return
        
        new_local_roles = []

        for lr in local_roles:
            if lr not in user_roles:
                new_local_roles.append(lr)

        if new_local_roles:
            container.manage_setLocalRoles(user_id, new_local_roles)
        
    def get_container(self):
        
        return self.aq_inner.aq_parent

    def sync(self):

        raise NotImplementedError 


InitializeClass(GroupManagement)


