import md5

from cStringIO import StringIO

from webdav.common import rfc1123_date

from OFS.Image import File, cookId
from OFS.content_types import guess_content_type

from PackageFileSystem import package_file_system, copy_stream, \
     copy_stream_consumer
from Products.Gideon.Objects.Namespace import *


class PackageFile(SimpleItem):

	"""

	Simple external file proxy
	
	"""
	 
	meta_type = 'Package File'		
	
	security = ClassSecurityInfo()
	
	__call__ = DTMLFile('ui/PackageFileRowView',globals())
	
	_properties = ()

	content_type = ''
	
	def __init__(self,
		     id,
		     title='',
		     file_type='',
		     file_format='',
		     platform='',
		     arch='',
		     py_version=''):
		
		self.id = id
		self.title = title
		self.file_type = file_type
		self.file_format = file_format
		self.platform = platform
		self.arch = arch
		self.py_version = py_version

	security.declarePublic('view_checksum')
	def view_checksum(self):
		""" """
		return self.check_sum
	
	security.declarePrivate('update_checksum')
	def update_checksum(self):
		""" """

		CheckSum = md5.new()

		inputstream = self.get_file().input_stream()
		copy_stream_consumer(inputstream, CheckSum.update)

		self.check_sum = CheckSum.hexdigest()		

		return self.check_sum

	security.declarePrivate('file_upload')
	def file_upload(self, file, content_type=''):

		pkgid = self.get_package_info().package_name()

		# it would be a good idea to
		#size_check(file)
		
		d = package_file_system[pkgid]
		d.add_file_object(self.id, file)

		self.update_checksum()

		self.content_type = guess_content_type(self.id, '', None)

	security.declarePrivate('get_file')
	def get_file(self):
		
		pkgid = self.get_package_info().package_name()		
		d = package_file_system[pkgid]
		return d.get_file(self.id)

	security.declareProtected('View', 'get_size')
	def get_size(self):

		return self.get_file().size()
 
	security.declareProtected('View', 'get_content_type')
	def get_content_type(self):
		
		return self.content_type or 'application/octet-stream'

	def __str__(self):

		return self.get_file().input_stream().read()

	security.declareProtected('View', 'index_html')
	def index_html(self, REQUEST):
		
		"""

		return the file with it's corresponding MIME-type

		"""
		# HTTP If-Modified-Since header handling.
		# (Copied from OFS/Image.py) by way of extfile

		header = REQUEST.get_header('If-Modified-Since', None)
		if header is not None:
			header = string.split(header, ';')[0]
			try:    mod_since = long(DateTime(header).timeTime())
			except: mod_since = None
			if mod_since is not None:
				if self._p_mtime:
					last_mod = long(self._p_mtime)
				else:
					last_mod = long(0)
				if last_mod > 0 and last_mod < mod_since:
					RESPONSE.setStatus(304)
					return ''
					
		size = self.get_size()

		f = self.get_file()
		output = StringIO()		
		copy_stream(f.input_stream(), output)

		if REQUEST is not None:

			last_mod = rfc1123_date(self._p_mtime)
			
			REQUEST.RESPONSE.setHeader('Last-Modified',
						   last_mod)
			REQUEST.RESPONSE.setHeader('Content-Type',
						   self.get_content_type()[0])
						   
			REQUEST.RESPONSE.setHeader('Content-Length',
						   size)


		self.get_package().get_download_counter().change(1)
		
		return output.getvalue()



